from typing import List

from ._PickupAreaSector import PickupAreaSector


class PickupArea(object):
    def __init__(
        self,           # type: PickupArea
        pickup_name,    # type: str
        sectors,        # type: List[PickupAreaSector]
    ):
        # type: (...) -> None

        self.pickup_name = pickup_name
        self.sectors = sectors
