from sysactions import action
from actions.order.close_active_order_and_pause_user import close_active_order_and_pause_user
from .. import logger


@action
def handle_power_off_screen_mobile(
    pos_id,     # type: str
):
    # type: (...) -> None

    logger.debug("Call handle power off screen mobile")
    close_active_order_and_pause_user(
        pos_id=pos_id,
        void_reason='POS_MOBILE_ON_PAUSE',
    )
