class Course(object):
    def __init__(self, course_id, name, description, abbreviation, delay):
        # type: (int, str, str, str, int) -> None

        self.course_id = course_id
        self.name = name
        self.description = description
        self.abbreviation = abbreviation
        self.delay = delay
