class UpdateMediaJson(object):
    def __init__(self):
        # type: () -> None

        self.catalog = {}
        self.navigation = {}
        self.banners_totem = {}
        self.images_totem = {}
        self.videos_totem = {}
