from typing import List


class GetOperatorsLogoutSessionsResponse(object):

    def __init__(
        self,                       # type: GetOperatorsLogoutSessionsResponse
        business_period,            # type: str
        operators_logout_sessions,  # type: List[str]
    ):
        # type: (...) -> None

        self.business_period = business_period
        self.operators_logout_sessions = operators_logout_sessions
