from ._OrderException import OrderException


class ProductInRupture(OrderException):

    def __init__(
        self,       # type: ProductInRupture
        part_code,  # type: int
    ):
        # type: (...) -> None
        super(ProductInRupture, self).__init__(30, str(part_code))

        self.part_code = part_code
