from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Sequence,
)

from nreports_api.shared import JSON


@dataclass
class Configs(object):

    database_dir: str
    report_files: Dict[str, List[JSON]]
    builder_extra_modules: Sequence[str]
    images_path: Dict[str, str]
    constants: Dict[str, str]
    component_name: str = "nReports"
