class CodedError(Exception):

    def __init__(
        self,       # type: CodedError
        code,       # type: int
        message,    # type: str
    ):
        # type: (...) -> None

        self.code = code
        self.message = message


class OrderError(object):

    PRODUCT_IN_RUPTURE = CodedError(
        code=40,
        message="Product is ruptured",
    )
