from helper import ExtendedEnum
from messagebus import TokenPriority, TokenCreator

MESSAGE_GROUP = "38"


class ListenedMessages(ExtendedEnum):
    TK_DAILY_GOALS_UPDATE_GOALS = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "1")
    TK_DAILY_GOALS_UPDATE_TOTAL_SOLD = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "2")
    TK_DAILY_GOALS_TERMINATE_REQUEST = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "3")
    TK_DAILY_GOALS_GET_SALES_CHALLENGE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "4")
    TK_DAILY_GOALS_INSERT_SALES_CHALLENGE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "5")
    TK_DAILY_GOALS_DELETE_SALES_CHALLENGE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "6")
    TK_DAILY_GOALS_UPDATE_SALES_CHALLENGE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "7")
